#! /usr/bin/python
# -*- coding: utf-8 -*-
#  Copyright (C) 2006, 2007, 2008, 2010 Stewart Adam
#  This file is part of fwfstab.

#  fwfstab is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  fwfstab is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with fwfstab; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import getopt
import gtk
import gobject
import os
import pango
import sys
#--
import fwfstab
from fwfstab.const import INSTALL_DIR, UID, OPTIONS
from fwfstab.i18n import _
from fwfstab import disks
from fwfstab import fstab
from fwfstab import interface
from fwfstab import widgets

def reportBug(etype=None, evalue=None, tb=None):
  """Report a bug dialog"""
  import traceback
  c = interface.Controller(os.path.join(INSTALL_DIR, 'BugReport.glade'), 'bugreport')
  if not etype and not evalue and not tb:
    (etype, evalue, tb) = sys.exc_info()
  tracebackText = ''.join(traceback.format_exception(etype, evalue, tb))
  reportWindow = widgets.bugReport(c.ui.bugreport, c.ui.bugreportTextview, None, tracebackText)
  response = reportWindow.runAndDestroy()
  if response == gtk.RESPONSE_OK:
    filename = widgets.saveFilename(c.ui.bugreport)
    if not filename:
      sys.exit(1)
    if fwfstab.CheckPerms(filename):
      import datetime
      fh = open(filename, 'w')
      fh.write(_('fwfstab v%s bug report written saved at %s') % (fwfstab.__version__, datetime.datetime.today().strftime('%I:%M %p on %Y-%m-%d\n')))
      fh.write(tracebackText)
      fh.close()
      sys.exit(1)
    else:
      print _('Could not write bug report due to insufficient permissions.')
      sys.exit(1)
  elif response == gtk.RESPONSE_CLOSE:
    sys.exit(1)
  print '%s: %s' % (etype, evalue)

sys.excepthook = reportBug

class fwfstabCallbacks(interface.Controller):
  def __init__(self, fstabFile, useAutodetect=True):
    """Initialize a new instance.
    `ui' is the user interface to be controlled."""
    # setup
    interface.Controller.__init__(self, os.path.join(INSTALL_DIR, 'fwfstab.glade'), 'main')
    self.fstabFile = os.path.abspath(fstabFile)
    self.useAutodetect = disks.autodetect_capable
    # start doing stuff here
    self.setTitle(self.fstabFile)
    # transient for...
    self.ui.about.set_transient_for(self.ui.main)
    self.ui.error.set_transient_for(self.ui.main)
    self.ui.confirm.set_transient_for(self.ui.main)
    self.ui.editor.set_transient_for(self.ui.main)
    self.ui.options.set_transient_for(self.ui.main)
    # set version in About
    self.ui.aboutVersionLabel.set_text('<span size="xx-large" weight="bold">fwfstab %s</span>' % fwfstab.__version__)
    self.ui.aboutVersionLabel.set_use_markup(True)
    # icons
    appIcon = os.path.join(INSTALL_DIR, 'fwfstab.png')
    if os.path.exists(appIcon):
      self.ui.aboutAppImage.set_from_file(appIcon)
      self.ui.main.set_icon_from_file(appIcon)
      self.ui.editor.set_icon_from_file(appIcon)
    # Widget setup
    self.statusbar = widgets.StatusBar(self.ui.statusbar1)
    # Entry treeview setup - last gobject.TYPE_STRING is for comment (hidden)
    liststore = gtk.ListStore(gobject.TYPE_INT, gobject.TYPE_STRING,
                              gobject.TYPE_STRING, gobject.TYPE_STRING,
                              gobject.TYPE_STRING, gobject.TYPE_STRING,
                              gobject.TYPE_STRING, gobject.TYPE_STRING,)
    cell = gtk.CellRendererText()
    col = gtk.TreeViewColumn("Device", cell, text=1)
    col.set_sort_column_id(0)
    col.set_resizable(True)
    self.ui.mainEntriesTreeview.append_column(col)
    cell = gtk.CellRendererText()
    col = gtk.TreeViewColumn("Mount Point", cell, text=2)
    col.set_sort_column_id(1)
    col.set_resizable(True)
    self.ui.mainEntriesTreeview.append_column(col)
    cell = gtk.CellRendererText()
    col = gtk.TreeViewColumn("Filesystem", cell, text=3)
    col.set_sort_column_id(2)
    col.set_resizable(True)
    self.ui.mainEntriesTreeview.append_column(col)
    cell = gtk.CellRendererText()
    col = gtk.TreeViewColumn("Mount Options", cell, text=4)
    cell.set_property('ellipsize', pango.ELLIPSIZE_END)
    col.set_sort_column_id(3)
    col.set_resizable(True)
    # makes it take as much space as possible
    col.set_expand(True)
    self.ui.mainEntriesTreeview.append_column(col)
    cell = gtk.CellRendererText()
    col = gtk.TreeViewColumn("Freq", cell, text=5)
    col.set_sort_column_id(4)
    col.set_resizable(False)
    col.set_max_width(100)
    self.ui.mainEntriesTreeview.append_column(col)
    cell= gtk.CellRendererText()
    col = gtk.TreeViewColumn("Pass No.", cell, text=6)
    col.set_sort_column_id(5)
    col.set_resizable(False)
    col.set_max_width(100)
    self.ui.mainEntriesTreeview.append_column(col)
    self.ui.mainEntriesTreeview.set_model(liststore)
    # Options treeview setup
    widgets.setupOptionsTreeview(self.ui.optionsTreeview)
    # drives combobox/liststore
    liststore = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
    self.ui.editorDevicePreconfiguredCombobox.set_model(liststore)
    cell = gtk.CellRendererText()
    self.ui.editorDevicePreconfiguredCombobox.clear()
    self.ui.editorDevicePreconfiguredCombobox.pack_start(cell, True)
    self.ui.editorDevicePreconfiguredCombobox.add_attribute(cell, 'text', 1)
    liststore = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
    self.ui.editorDevicePreconfigured1Combobox.set_model(liststore)
    cell = gtk.CellRendererText()
    self.ui.editorDevicePreconfigured1Combobox.clear()
    self.ui.editorDevicePreconfigured1Combobox.pack_start(cell, True)
    self.ui.editorDevicePreconfigured1Combobox.add_attribute(cell, 'text', 1)
    # do we autodetect drives?
    if self.useAutodetect:
      self._setDevicePreconfigured(True)
      self._setFilesystemAutomatic(True)
    else:
      self.ui.editorDeviceRadio2.set_active(True)
      self.ui.editorFilesystemRadio2.set_active(True)
      #self._setDevicePreconfigured(False)
      #self._setFilesystemAutomatic(False)
      self.ui.editorDeviceRadio1.set_sensitive(False)
      self.ui.editorFilesystemRadio1.set_sensitive(False)
    # Refresh data
    self.refreshMountOptions()
    if not self.refreshFstab():
      sys.exit(0)

  def displayError(self, parent, header, message):
    """Display the error window and set the labels"""
    self.ui.errorHeaderLabel.set_text('<span size="x-large" weight="bold">%s</span>' % header)
    self.ui.errorHeaderLabel.set_use_markup(True)
    self.ui.errorMessageLabel.set_text(message)
    self.ui.errorMessageLabel.set_use_markup(True)
    dialog = widgets.GenericDia(self.ui.error, _("Error"), parent)
    dialog.run()
    dialog.destroy()
  
  def displayConfirm(self, parent, header, message):
    """Display the confirm window and set the labels"""
    self.ui.confirmHeaderLabel.set_text('<span size="x-large" weight="bold">%s</span>' % header)
    self.ui.confirmHeaderLabel.set_use_markup(True)
    self.ui.confirmMessageLabel.set_text(message)
    self.ui.confirmMessageLabel.set_use_markup(True)
    dialog = widgets.GenericDia(self.ui.confirm, _("Please confirm"), parent)
    retval = dialog.run()
    dialog.destroy()
    return retval

  def askToSaveUnchangedChanges(self):
    """Asks the user to save any unsaved changes"""
    retval = self.displayConfirm(self.ui.main, _('Would you like to save changes?'), _('If you do not, any changes you have made will be discarded.'))
    if retval == gtk.RESPONSE_YES:
      self.saveChanges()

  # WRAPPERS
  def quit(self, widget=None, event=None):
    """Quit the program"""
    if self.changes == True:
      self.askToSaveUnchangedChanges()
    try:
      gtk.main_quit()
    except RuntimeError, msg:
      pass
    return False

  def hide(self, widget, event=None):
    """Used to hide a close non-destructively"""
    widget.hide()
    return True

  def refreshFstab(self):
    """Refreshes fstab entries in the treeview."""
    try:
      newEntries = fstab.read(self.fstabFile)
    except IOError: # can't open file
      self.displayError(self.ui.main, _("Could not read %s") % self.fstabFile, _("Please ensure that it still exists and that you have read permissions."))
      return False
    # Start the refresh now that we are sure we can read the file
    model = self.ui.mainEntriesTreeview.get_model()
    model.clear()
    self.fstabEntries = {}
    linenum = 1
    for fstabLine in newEntries:
      # Store it in our dictionary by line number
      self.fstabEntries[linenum] = fstabLine
      # Show it in the Treeview if it's parsable and not a comment
      if fstabLine.is_parsable():
        if not fstabLine.is_comment_or_whitespace():
          model.append([linenum]+list(fstabLine.get_all_fields()))
      else:
        self.displayError(self.ui.main, _('Could not parse entry'), _('Could not parse the following line in the fstab:\n%s\nfwfstab will ignore this line and write it back to the fstab as-is when saving.') % fstabLine.get_raw_entry_text())
      linenum += 1
    return True

  def refreshMountOptions(self):
    """Refreshes the mount options window with contents of OPTIONS"""
    model = self.ui.optionsTreeview.get_model()
    model.clear()
    for option in OPTIONS:
      model.append([option[0], option[1]])

  def refreshDrives(self):
    """Refreshes available disks"""
    if not self.useAutodetect:
      return False
    model = self.ui.editorDevicePreconfiguredCombobox.get_model()
    model.clear()
    drives = disks.getHardDrives()
    if not drives:
      print _('No drives found!')
    for drive in drives:
      print _('Drive found: %(a)s: %(b)s') % {'a': drive.device.path, 'b': drive.device.model}
      model.append([drive, '%s: %s' % (drive.device.path, drive.device.model)])

  def saveChanges(self):
    """Write changes to the fstab"""
    def saveRow(model, path, iter, fstabEntries):
      lineno, fs_spec, fs_file, fs_vfstype, fs_mntops, fs_freq, fs_passno, end_comments = model[iter]
      if fs_file == '/' and fs_passno != '0':
        fs_passno = '2'
      fstabEntries[lineno].set_all_fields(fs_spec, fs_file, fs_vfstype, fs_mntops, fs_freq, fs_passno, end_comments)
    # --
    self.ui.mainEntriesTreeview.get_model().foreach(saveRow, self.fstabEntries)
    linenums = self.fstabEntries.keys()
    linenums.sort()
    retval = fstab.write(self.fstabFile, [self.fstabEntries[linenum] for linenum in linenums])
    if retval == -1:
      self.displayError(self.ui.main, _('Could not save fstab'), _('A backup of the %s could not be created.') % self.fstabFile)
      return
    elif retval == 0:
      self.displayError(self.ui.main, _('Could not save fstab'), _('%s could not be written to.\nPlease make sure you have write permissions on the file.') % self.fstabFile)
      return
    self.setTitle(self.fstabFile, changes=False)
    self.statusbar.newmessage(_('Saved changes to %s') % self.fstabFile)

  def setTitle(self, fstabFile, changes=False):
    """Sets the window title"""
    self.changes = changes
    if self.changes:
      prefix = '*'
    else:
      prefix = ''
    # shortens filename if too long
    dirName, fileName = os.path.split(self.fstabFile)
    if not dirName:
      dirName = os.getcwd()
    if len(dirName) > 40:
      dirName = '%s...%s' % (dirName[:15], dirName[-15:])
    self.ui.main.set_title('%s%s (%s) - fwfstab' % (prefix, fileName, dirName))

  def _setLocked(self, bool):
    """(Un)Lock the UI"""
    self.ui.mainVButtonBox.set_sensitive(not bool)
    self.ui.mainEntriesTreeview.set_sensitive(not bool)

  # MENUS
  ## File menu
  def on_open1_activate(self, widget):
    """Open new fstab file"""
    if self.changes == True:
      self.askToSaveUnchangedChanges()
    pathBrowser = widgets.PathBrowser(self.ui.chooser, _("Select a file"), self.ui.main)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
    pathBrowser.set_select_multiple(False)
    response = pathBrowser.run()
    path = pathBrowser.get_filename()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    self.fstabFile = os.path.abspath(path)
    if not self.refreshFstab():
      return False
    self.setTitle(self.fstabFile)

  def on_save_as1_activate(self, widget):
    """Save fstab file as..."""
    pathBrowser = widgets.PathBrowser(self.ui.chooser, _("Select a file"), self.ui.main)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
    pathBrowser.set_select_multiple(False)
    response = pathBrowser.run()
    path = pathBrowser.get_filename()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    self.fstabFile = os.path.abspath(path)
    self.setTitle(self.fstabFile)
    self.saveChanges()

  def on_apply_changes1_activate(self, widget):
    """Save Changes"""
    self.saveChanges()

  def on_quit1_activate(self, wiget):
    """Destroy main window, quit program"""
    self.ui.main.destroy()

  ## Help menu
  def on_about1_activate(self, widget):
    """Show about window"""
    self.ui.about.show()

  def on_fstab_options1_activate(self, widget):
    """Show options window"""
    self.ui.options.show()

  # ABOUT WINDOW
  def on_aboutCloseButton_clicked(self, widget):
    """About window Close button"""
    self.ui.about.hide()

  # OPTIONS WINDOW
  def on_optionsCloseButton_clicked(self, widget):
    """Close options window"""
    self.ui.options.hide()

  # EDITOR WINDOW
  def on_editorHelpButton_clicked(self, widget):
    """Editor Help button"""
    self.ui.options.show()

  def on_editorApplyButton_clicked(self, widget):
    """Editor OK button"""
    self.closeEditor()

  def on_editorCancelButton_clicked(self, widget, event=None):
    """Editor Close button"""
    self.ui.editor.hide()
    ##self.action = None
    self._setLocked(False)

  def showEditor(self, action=None):
    """Show detailed editor, clearing or filling in fields"""
    self.ui.editorNotebook.set_current_page(0)
    self.action = action
    self.refreshDrives()
    self.ui.editorDevicePreconfiguredCombobox.set_active(0)
    self.ui.editorDevicePreconfigured1Combobox.set_active(0)
    if self.action == None:
      self.displayError(self.ui.main, _('Internal Error'), _("Please try to add/edit the entry again.\nIf the problem persists, please report a bug."))
      return
    elif self.action == 'add':
      # clean out
      if self.useAutodetect:
        self.ui.editorDeviceRadio1.set_active(True)
      else:
        self.ui.editorDeviceRadio2.set_active(True)
      self.ui.editorDeviceManualEntry.set_text('')
      if self.useAutodetect:
        self.ui.editorFilesystemRadio1.set_active(True)
      else:
        self.ui.editorFilesystemRadio2.set_active(True)
      self.ui.editorFilesystemManualEntry.set_text('')
      self.ui.editorMountpointEntry.set_text('')
      self.ui.editorDumpFSCheck.set_active(False)
      self.ui.editorCheckFSCheck.set_active(False)
    elif self.action == 'edit':
      model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
      if iterator == None:
        self.statusbar.newmessage(_('Please select a row'))
        return
      linenum, node, mountPoint, fsType, options, dump, check, end_comment = model[iterator]
      self.ui.editorDeviceRadio2.set_active(True)
      self.ui.editorFilesystemRadio2.set_active(True)
      self.ui.editorDeviceManualEntry.set_text(node)
      self.ui.editorFilesystemManualEntry.set_text(fsType)
      self.ui.editorMountpointEntry.set_text(mountPoint)
      self.ui.editorOptionsEntry.set_text(options)
      if dump == '0':
        self.ui.editorDumpFSCheck.set_active(False)
      else:
        self.ui.editorDumpFSCheck.set_active(True)
      if check == '0':
        self.ui.editorCheckFSCheck.set_active(False)
      else:
        self.ui.editorCheckFSCheck.set_active(True)
    self._setLocked(True)
    self.ui.editor.show()

  def closeEditor(self):
    """Hide Editor window, saving changes or adding new entry"""
    if self.action == None:
      self.displayError(self.ui.main, _("Internal Error"), _("Please try to add/edit the entry again.\nIf the problem persists, please report a bug."))
      return False
    model, treeIter = self.ui.mainEntriesTreeview.get_selection().get_selected()
    if self.ui.editorDeviceRadio1.get_active():
      nodeModel = self.ui.editorDevicePreconfigured1Combobox.get_model()
      iter = self.ui.editorDevicePreconfigured1Combobox.get_active_iter()
      node = disks.getPartitionNodeName(nodeModel.get_value(iter, 0))
    elif self.ui.editorDeviceRadio2.get_active():
      node = self.ui.editorDeviceManualEntry.get_text()
    if self.ui.editorFilesystemRadio1.get_active():
      fsType = self.ui.editorFilesystemAutodetectLabel.get_text()
    elif self.ui.editorFilesystemRadio2.get_active():
      fsType = self.ui.editorFilesystemManualEntry.get_text()
    mountPoint = self.ui.editorMountpointEntry.get_text()
    options = self.ui.editorOptionsEntry.get_text()
    if self.ui.editorCheckFSCheck.get_active():
      check = '1'
    else:
      check = '0'
    if self.ui.editorDumpFSCheck.get_active():
      dump = '1'
    else:
      dump = '0'
    if not node:
      self.displayError(self.ui.editor, _('Device field is empty'), _('Please fill in all of the fields.'))
      return False
    elif not mountPoint:
      self.displayError(self.ui.editor, _('Mount point field is empty'), _('Please fill in all of the fields.'))
      return False
    elif not fsType:
      self.displayError(self.ui.editor, _('Filesystem field is empty'), _('Please fill in all of the fields.'))
      return False
    # now do stuff
    model = self.ui.mainEntriesTreeview.get_model()
    if self.action == 'add':
      comment = ''
      linenum = max(self.fstabEntries.keys())+1
      line = ' '.join([node, mountPoint, fsType, options, dump, check, comment]).strip()
      self.fstabEntries[linenum] = fstab.fstabLine(line)
      model.append([linenum, node, mountPoint, fsType, options, dump, check, comment])
      self.setTitle(self.fstabFile, changes=True)
      self.statusbar.newmessage(_('Added entry `%s\''))
    elif self.action == 'edit':
      model.set_value(treeIter, 1, node)
      model.set_value(treeIter, 2, mountPoint)
      model.set_value(treeIter, 3, fsType)
      model.set_value(treeIter, 4, options)
      model.set_value(treeIter, 5, dump)
      model.set_value(treeIter, 6, check)
      self.setTitle(self.fstabFile, changes=True)
      self.statusbar.newmessage(_("Modified entry with mountpoint %s" % mountPoint))
    else:
      self.displayError(self.ui.editor, _('Internal Error'), _('Please report a bug: self.action has a value of `%s\' which is unhandled.' % self.action))
    self.ui.editor.hide()
    self.action = None
    self._setLocked(False)
    return True

  # MAIN WINDOW
  def on_mainNewEntryButton_clicked(self, widget):
    """Main window Add button"""
    self.showEditor('add')

  def on_mainEditEntryButton_clicked(self, widget):
    """Main window Edit button"""
    self.showEditor('edit')

  def on_mainEntriesTreeview_row_activated(self, widget, path, column):
    """User double-clicked on an entry"""
    self.on_mainEditEntryButton_clicked(None)

  # Remove
  def on_mainDeleteEntryButton_clicked(self, widget):
    model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
    if iterator == None:
      self.statusbar.newmessage(_('Please select a row'))
      return
    linenum = model.get_value(iterator, 0)
    drive = model.get_value(iterator, 1)
    path = model.get_value(iterator, 2)
    self.fstabEntries.pop(linenum)
    self.statusbar.newmessage(_("Removed entry `%(a)s' with mountpoint `%(b)s'" % {'a': drive, 'b': path}))
    model.remove(iterator)

  def on_mainMountButton_clicked(self, widget):
    """Mounts the selected drive"""
    if self.changes == True:
      retval = self.displayConfirm(self.ui.main, _('Unsaved Changes'), _('(Un)mounting a drive may not work until changes are applied.\nWould you like to save now?'))
      if retval == gtk.RESPONSE_YES:
        self.saveChanges()
    model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
    if iterator == None:
      self.statusbar.newmessage(_('Please select a row'))
      return
    drive = model.get_value(iterator, 1)
    path = model.get_value(iterator, 2)
    fstype = model.get_value(iterator, 3)
    
    if fstype == "swap":
      executable = "swapon"
      argument = drive
    else:
      executable = "mount"
      argument = path
    if not fwfstab.which(executable):
      self.displayError(self.ui.main, _("Could not mount filesystem"), _("The executable `%s' could not be found!") % executable)
      return False
    retval, stdout, stderr = fwfstab.execute([fwfstab.which(executable), argument])
    if retval == 0:
      self.statusbar.newmessage(_('Mounted drive `%(a)s\' with mountpoint `%(b)s\'' % {'a': drive, 'b': path}))
    else:
      self.displayError(self.ui.main, _('Could not mount filesystem'), stderr.read())
    stdout.close()
    stderr.close()

  def on_mainUnmountButton_clicked(self, widget):
    """Unmounts the selected drive"""
    if self.changes == True:
      retval = self.displayConfirm(self.ui.main, _('Unsaved Changes'), _('(Un)mounting a drive may not work until changes are applied. Would you like to save now?'))
      if retval == gtk.RESPONSE_YES:
        self.saveChanges()
    model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
    if iterator == None:
      self.statusbar.newmessage(_('Please select a row'))
      return
    drive = model.get_value(iterator, 1)
    path = model.get_value(iterator, 2)
    fstype = model.get_value(iterator, 3)
    
    if fstype == "swap":
      executable = "swapoff"
      argument = drive
    else:
      executable = "umount"
      argument = path
    if not fwfstab.which(executable):
      self.displayError(self.ui.main, _("Could not unmount filesystem"), _("The executable `%s' could not be found!") % executable)
      return False
    retval, stdout, stderr = fwfstab.execute([fwfstab.which(executable), argument])
    if retval == 0:
      self.statusbar.newmessage(_('Unmounted drive `%(a)s\' with mountpoint `%(b)s\'' % {'a': drive, 'b': path}))
    else:
      self.displayError(self.ui.main, _('Could not unmount filesystem'), stderr.read())
    stdout.close()
    stderr.close()

  def _setDevicePreconfigured(self, bool):
    """Set """
    for i in ['editorDevicePreconfiguredCombobox',
              'label71',
              'editorDevicePreconfigured1Combobox']:
      getattr(self.ui, i).set_sensitive(bool)
    self.ui.editorDeviceManualEntry.set_sensitive(not bool)

  def _setFilesystemAutomatic(self, bool):
    self.ui.editorFilesystemAutodetectLabel.set_sensitive(bool)
    self.ui.editorFilesystemManualEntry.set_sensitive(not bool)

  def on_editorDeviceRadio1_toggled(self, widget):
    self._setDevicePreconfigured(True)
  def on_editorDeviceRadio2_toggled(self, widget):
    self._setDevicePreconfigured(False)
  def on_editorFilesystemRadio1_toggled(self, widget):
    self._setFilesystemAutomatic(True)
  def on_editorFilesystemRadio2_toggled(self, widget):
    self._setFilesystemAutomatic(False)

  def on_editorDevicePreconfiguredCombobox_changed(self, widget):
    """Drive selected: Populate xxx: drive.desc entries into combobox"""
    self.ui.editorFilesystemRadio1.set_active(True)
    model = self.ui.editorDevicePreconfiguredCombobox.get_model()
    active = self.ui.editorDevicePreconfiguredCombobox.get_active_iter()
    if not active:
      return
    drive = model.get_value(active, 0)
    partModel = self.ui.editorDevicePreconfigured1Combobox.get_model()
    partModel.clear()
    partitions = drive.partitions
    if not partitions:
      self.displayError(self.ui.editor, _('Could not locate partitions'), _('fwfstab could not autodetect partitions on the selected drive.'))
    print _('Using drive: %(a)s: %(b)s') % {'a': drive.device.path, 'b': drive.device.model}
    for partition in partitions:
      print _('--> Partition found: %(a)s, %(b)s, %(c)sMB') % {'a': disks.getPartitionNodeName(partition),
                                                               'b': disks.getPartitionType(partition),
                                                               'c': partition.getSize()}
      if not disks.getPartitionIsMountable(partition):
        continue
      size = partition.getSize()
      suffix = _('MB')
      if size >= 1024:
        size = round(size/1024, 2)
        suffix = _('GB')
      partModel.append([partition, '%s (%s%s)' % (disks.getPartitionNodeName(partition),
                                                  size, suffix)
                      ])
    self.ui.editorDevicePreconfigured1Combobox.set_active(0)
    
  def on_editorDevicePreconfigured1Combobox_changed(self, widget):
    """Partition selected: Populate /dev/xxx (XX.XXMB|GB) entries into combobox"""
    self.ui.editorFilesystemRadio1.set_active(True)
    model = self.ui.editorDevicePreconfigured1Combobox.get_model()
    active = self.ui.editorDevicePreconfigured1Combobox.get_active_iter()
    if active:
      partition = model.get_value(active, 0)
      # manual
      self.ui.editorDeviceManualEntry.set_text(disks.getPartitionNodeName(partition))
      # part type
      self.ui.editorFilesystemAutodetectLabel.set_text(disks.getPartitionType(partition))

  def on_editorMountPointBrowseButton_clicked(self, widget):
    """Browse for a directory and fill in the path"""
    pathBrowser = widgets.PathBrowser(self.ui.chooser, _("Select a folder"), self.ui.editor)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    pathBrowser.set_select_multiple(False)
    response = pathBrowser.run()
    path = pathBrowser.get_filename()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    self.ui.editorMountpointEntry.set_text(path)

def usage():
    """Print the application usage"""
    print _("""Usage:\tfwfstab [OPTIONS] [FILE]
Valid Options:
\t-h, --help\n\t\tPrint this message and exit
\t-a, --autodetect\n\t\tTry to autodetect drives (default, root privledges required)
\t-n, --no-autodetect\n\t\tDon't try to autodetect drives
FILE is assummed to be /etc/fstab when omitted.""")

# Only if we're in main execution
if __name__ == "__main__":
  autodetect = True
  try:
    avalableOptions = ["help", "autodetect", "no-autodetect"]
    # letter = plain options
    # letter: = option with arg
    (opts, files) = getopt.getopt(sys.argv[1:],"han", avalableOptions)
  except (getopt.GetoptError), errorDesc:
    print errorDesc
    usage()
    sys.exit(1)
  # Parse args, take action
  if opts:
    for (opt, value) in opts:
      if opt == "-h" or opt == "--help":
        usage()
        sys.exit(1)
      elif opt == "-a" or opt == "--autodetect":
        autodetect = True
      elif opt == "-n" or opt == "--no-autodetect":
        autodetect = False
  
  if not files:
    fstabFile = "/etc/fstab"
  else:
    fstabFile = files[0]
  
  # Startup the application and call the gtk event loop
  try:
    # We only support editing one file at a time use files[0]
    # Default to /etc/fstab
    if not os.path.exists(fstabFile):
      print _("Error: No such file or folder %s." % fstabFile)
      sys.exit(1)
    elif not os.path.isfile(fstabFile):
      print _("Error: %s is not a file." % fstabFile)
      sys.exit(1)
    mainApp = fwfstabCallbacks(fstabFile, autodetect)
    gtk.main()
  except KeyboardInterrupt:
    # ctrl+c?
    mainApp.quit(None)

